package Class::SingletonMethod::Hash;

use 5.006002; # encourage upgrade
use strict;
use warnings;
use Carp;
use Data::Dumper;

our $VERSION = sprintf "%d.%02d", q$Revision: 2.0 $ =~ /(\d+)/g;
our $DEBUG   = 0;

my %Method;

sub singleton_method {
    my $self = shift;
    $self->isa('HASH') 
	or croak "sorry. $self must be a blessed hash reference";
    my ( $method, $subref ) = @_;
    ref $subref eq 'CODE'
      or croak '$obj->singleton_method(name => sub{...})';
    $self->{_meta}{method}{$method} = $subref;
}
*add_method = \&singleton_method;

sub remove_method{
    my $self = shift;
    my $method = shift;
    return unless $self->{_meta}{method};
    delete $self->{_meta}{method}{$method};
}

sub can {
    my ( $self, $method ) = @_;
    my $subref = $self->{_meta}{method}{$method};
    return $subref if $subref;
    return $self->SUPER::can($method);
}

sub DESTROY {
    my $self = shift;
    $DEBUG and carp "Destroing $self";
}

sub AUTOLOAD {
    my $self   = shift;
    my $method = our $AUTOLOAD;
    $method =~ s/.*:://;
    $DEBUG and carp "($self)->$method";
    if ( my $subref = $self->{_meta}{method}{$method} ){
        $subref->( $self, @_ );
    }
    else {
        my $pkg = ref $self;
        croak qq(Can't locate object method "$method" via package "$pkg");
    }
}

1;

__END__

=head1 NAME

Class::SingletonMethod::Hash - Store singleton methods in objects themselves

=head1 SYNOPSIS

    use base 'Class::SingletonMethod::Hash';
    my $a = Some::Class->new; 
    my $b = Some::Class->new; 
     
    $a->singleton_method( dump => sub { 
      my $self = shift; 
      require Data::Dumper; 
      print STDERR Date::Dumper::Dumper($self)  
    }); 
     
    $a->dump; # Prints a representation of the object. 
    $b->dump; # Can't locate method "dump" 

=head1 DESCRIPTION

This module provides a Perl implementation of singleton methods which
is further discussed in L<Class::SingletonMethod>.

This module implements singleton methods almost the same way as
L<Class::SingletonMethod::Std> but instead of storing methods
in inside-out class manner, it stores in
 C<< $self->{_meta}{method}{$method} >>, which is kind of Perl6ish.

The advantage of this apporach is that you can serialize the singleton
methods like this.

  use Storable qw/freeze thaw/;
  # insert the code above
  local($Storable::Deparse) = 1;
  my $serialized_a = freeze($a);
  my $restored_a   = thaw($serialized_a);
  $restored_a->dump; # works!

=head1 AUTHOR

Simon Cozens, C<simon@cpan.org>

Dan Kogai, C<dankogai@dan.co.jp>

=head1 SEE ALSO

ruby(1)

Advanced Perl Programming, 2nd ed.
L<http://www.oreilly.com/catalog/advperl2/>

=cut
